// Agent.cpp: implementation of the CAgent class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Agent.h"
#include "StrategySpace.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAgent::CAgent(int q, int m, int T, double tau, long *idum)
{
	int i;

	m_iQ = q;							//store values
	m_pidum = idum;
	m_bSSpace = false;

	m_dThres = tau * T;					//points threshold

	m_pStrats = new CStrategy*[m_iQ];	//create strategies
	for(i=0;i<m_iQ;i++)
		m_pStrats[i] = new CStrategy(m, T, idum);

	work = new int[m_iQ];


	// Rather than choosing between the pred and points function to obtain
	// the prediction or # strategy points based on the m_bSSpace variable,
	// it would be quicker to define a pointer to the appropriate function.
	// Or more specificially, a functor would do the job
	// see http://function-pointer.org after a slight modification of the 
	// CStrategy class

}

//strategy space implementation
CAgent::CAgent(int q, int T, double tau, CStrategySpace *psspace, long *idum)
{
	int i, ssize;

	m_iQ = q;							//store values
	m_pidum=idum;
	m_bSSpace = true;
	m_pSSpace = psspace;

	ssize = m_pSSpace->ssize();

	m_dThres = tau * T;					//points threshold

	m_iStrats = new int[m_iQ];			//create strategies
	for(i=0;i<m_iQ;i++)
		m_iStrats[i] = (int)(ssize*ran2(m_pidum));

	work = new int[m_iQ];

}


CAgent::~CAgent()
{
	delete [] work;
	delete [] m_pStrats;
	if(m_bSSpace)
		delete [] m_iStrats;
}


//////////////////////////////////////////////////////////////////////
// Game
//////////////////////////////////////////////////////////////////////


//determine best active strategy - either strategy index or -1 if all inactive
int CAgent::bastrat()
{
	int i,nas,S,max;
	
	nas=0;		//number of active strategies (<=m_iQ)
	max=0;		//maximum number of virtual points

	//iterate through the strategies
	for(i=0;i<m_iQ;i++) {

		if(m_bSSpace)	//get the number of virtual points for the i'th strategy
			S = m_pSSpace->points(m_iStrats[i]);
		else
			S = m_pStrats[i]->points();	

		if(S >= m_dThres) {			//determine if strategy is active
			if(S > max) {			//now check if it's the best
				nas=1;
				max=S;
				work[0]=i;
			}
			else if(S==max) {		//tied situation
				work[nas]=i;
				nas++;
			}
		}
	}

	//now determine best active strategy
	if(nas==0) 
		return -1;		//no active strategy
	else if(nas==1)
		return work[0];	//distinct active strategy
	else {				//must be a tie-break situation
		i=(int)(nas*ran2(m_pidum));		//want an integer between 0 and nas-1
		return work[i];
	}

}

//request a prediction based on the present history mu
int CAgent::pred(int mu)
{
	int bas = bastrat();				//determine best strategy

	if (bas==-1)
		return -1;						//no active strategy
	else
		if(m_bSSpace)					//strategy prediction
			return m_pSSpace->pred(m_iStrats[bas],mu);
		else
			return m_pStrats[bas]->pred(mu);	
}

//update strategy points
void CAgent::spoints_update(int mu, int win)
{
	for(int i=0;i<m_iQ;i++)
		m_pStrats[i]->points_update(mu, win);

	return;
}
