// BitArray.cpp: implementation of the CBitArray class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BitArray.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CBitArray::CBitArray(int Size)
{
    ArraySize = ( Size + IntSz - 1) / IntSz;
    TheArray = new int [ArraySize]; 
    N = Size;
    ClearAllBits();
}

CBitArray::~CBitArray()
{
	delete [] TheArray;
}



//******************************************************************
// IsInRange () - Check if bit is in range
//******************************************************************
inline int CBitArray::IsInRange(int i) const
{
    //***************************************************************
    // each bracket, {, }, MUST get its own line, and must be 
    // placed under the conditional
    //***************************************************************
    if (i < 0 || i >= N)
    {
        cerr << "Bit " << i << " exceeds size (" << N << ")" << endl;
        return 0;
    }
    return 1;
}

//******************************************************************
// SetBit () - Set a bit
//******************************************************************
void CBitArray::SetBit (int i)
{
    if (IsInRange(i))
        TheArray[i / IntSz] |= (1 << (i % IntSz));
}

//******************************************************************
// ClearBit () - Clear a bit
//******************************************************************
void CBitArray::ClearBit (int i)
{
    if (IsInRange(i)) {
		//i = i+2;
        TheArray[i / IntSz] &= ~(1 << (i % IntSz));
	}
}

//******************************************************************
// GetBit () - Get a bit
//******************************************************************
int CBitArray::GetBit (int i) const
{
    if (!IsInRange(i))
        return 0;

    return (TheArray[ i / IntSz] & (1 << (i % IntSz))) != 0;
}

//******************************************************************
// ClearAllBits () - Clear all the bits, if the code is more 
// involved than these functions, include some explanation of
// the routine.
//******************************************************************
void CBitArray::ClearAllBits()
{
    for (int i = 0; i < ArraySize; i++)
        TheArray[i] = 0;
}


