using System;
using SiEDataPointInterfacesLib;
using System.Runtime.InteropServices;

namespace SiEDataPoints_Managed.Value
{
	/// <summary>
	/// Summary description for DataPoint
	/// </summary>
	public struct DataPoint : IDataPoint, IComparable
	{
		public int CompareTo(object oRHS)
		{
			return this.m_Name.CompareTo(((DataPoint)oRHS).m_Name);
		}

		public int ID
		{
			get { return m_ID; }
			set { m_ID = value; }
		}
		public string Name
		{
			get { return m_Name; }
			set { m_Name = value; }
		}
		public int Channel 
		{
			get { return m_Channel;}
			set { m_Channel = value; }
		}

		private int m_ID;
		private string m_Name;
		private int m_Channel;
	}

	[Guid("8C10510C-82A4-39A6-8E4B-6FC4FBD8AF4C")]
	public class DataPoints : IDataPoints
	{
		private DataPoint[] m_coll;
		private int			m_size;

		public DataPoints()
		{
			m_size = 0;
		}


		public IDataPoint this[int index]
		{
			get { return m_coll[index]; }
		}

		public int Count
		{
			get { return m_size; }
		}
		
		public int Capacity
		{
			get
			{
				if (m_coll == null)
					return 0;
				return m_coll.Length; 
			}
			set
			{
				// Preserve the existing collection, if any
				DataPoint[] oldColl = m_coll;
				// Enforce a minimum capacity of 10
				int newCapacity = 10;
				if (value > newCapacity) newCapacity = value;
				m_coll = new DataPoint[newCapacity];

				// Copy the existing collection into the new collection
				m_size = 0;
				if (oldColl != null)
				{
					foreach(DataPoint pt in oldColl)
					{
						m_coll[m_size] = pt;
						++m_size;
					}
				}
			}
		}
		
		public void Add(IDataPoint pt)
		{
			// Make sure we have a collection to add to
			if (m_coll == null)
				Capacity = 10;
			// Increase capacity by 50% if necessary
			else if (m_size == m_coll.Length)
				Capacity = m_size + m_size/2;
			// Assign the new datapoint.  It won't be a value-type DataPoint
			// if a VB6 client is being used, because VB6 doesn't know how
			// to create one.  Therefore, copy the properties from the interface
			m_coll[m_size].ID = pt.ID;
			m_coll[m_size].Name = pt.Name;
			m_coll[m_size].Channel = pt.Channel;
			++m_size;
		}

		public void Sort()
		{
			Array.Sort(m_coll);
		}

		public IDataPoints FindPoints(string name, int channel)
		{
			if (name == null || name.Length == 0)
				return null;
			DataPoint pt = new DataPoint();
			pt.Name = name;
			int index = Array.BinarySearch(m_coll, pt);
			if (index < 0)
				return null;

			IDataPoints pts = (IDataPoints)new DataPoints();
			do 
			{
				if (m_coll[index].Name != name)
					break;
				if (channel > 0 && m_coll[index].Channel != channel)
				{
					++index;
					continue;
				}
				pts.Add((IDataPoint)m_coll[index]);
				++index;
			} while (index < m_size);
			return pts;
		}

	}
}
