// DataPoints.cpp : Implementation of CDataPoints

#include "stdafx.h"
#include "DataPoints.h"


// CDataPoints
//static
bool CDataPoints::operator <(const CAdapt<CComPtr<IDataPoint> > & refLHS, const CAdapt<CComPtr<IDataPoint> > & refRHS)
{
	CComBSTR bstrLHSName;
	if FAILED(refLHS.m_T->get_Name(&bstrLHSName))
		return false;
	CComBSTR bstrRHSName;
	if FAILED(refRHS.m_T->get_Name(&bstrRHSName))
		return false;
	return bstrLHSName < bstrRHSName;
}

STDMETHODIMP CDataPoints::get_Item(long lIndex, IDataPoint * * ppPoint)
{
	if ( (lIndex < 0) ||
			((unsigned long)lIndex >= m_coll.size()) )
			return E_INVALIDARG;
	return m_coll[lIndex].m_T.CopyTo(ppPoint);
}

STDMETHODIMP CDataPoints::get_Count(long * pVal)
{
	*pVal = (long)m_coll.size();
	return S_OK;
}

STDMETHODIMP CDataPoints::get_Capacity(long * pVal)
{
	*pVal = (long)m_coll.capacity();
	return S_OK;
}

STDMETHODIMP CDataPoints::put_Capacity(long pVal)
{
	m_coll.reserve(pVal);
	return S_OK;
}

STDMETHODIMP CDataPoints::Add(IDataPoint * pPoint)
{
	ATLASSERT(pPoint != NULL);
	m_coll.push_back(CAdapt<CComPtr<IDataPoint> >(pPoint));
	return S_OK;
}

STDMETHODIMP CDataPoints::Sort()
{
	// use the static member operator< overload to sort the collection
	std::sort(m_coll.begin(), m_coll.end(), CDataPoints::operator <);
	return S_OK;
}

STDMETHODIMP CDataPoints::FindPoints(BSTR bstrName, long lChannel, IDataPoints * * ppPoints)
{
	HRESULT hr = S_OK;

	CComPtr<IDataPoint> pPointToFind;
	hr = pPointToFind.CoCreateInstance(CLSID_DataPoint);
	if FAILED(hr)
		return hr;
	hr = pPointToFind->put_Name(bstrName);
	if FAILED(hr)
		return hr;

	std::pair<DataPointIterator, DataPointIterator> range = 
		std::equal_range(m_coll.begin(),
						 m_coll.end(),
						 CAdapt<CComPtr<IDataPoint> >(pPointToFind), CDataPoints::operator <);
	if (range.first == m_coll.end())
		return E_FAIL;
	CComBSTR bstrFirstName;
	hr = range.first->m_T->get_Name(&bstrFirstName);
	if FAILED(hr)
		return hr;
	if (bstrFirstName != bstrName)
		return E_FAIL;

	DataPointVector::difference_type count = range.second - range.first;
	if (count == 0)
		return E_FAIL;

	CComPtr<IDataPoints> pColl;
	hr = pColl.CoCreateInstance(CLSID_DataPoints);
	if FAILED(hr)
		return hr;

	bool bFound = false;
	for (DataPointIterator it = range.first; it != range.second; ++it)
	{
		if (lChannel != 0)
		{
			long lPtChannel;
			(*it).m_T->get_Channel(&lPtChannel);
			if (lPtChannel != lChannel)
				continue;
		}

		IDataPoint * pPoint;
		(*it).m_T.CopyTo(&pPoint);
		pColl->Add(pPoint);
		bFound = true;
	}
	if (!bFound)
		return E_FAIL;

	*ppPoints = pColl.Detach();
	return S_OK;
}
