// DataPoints.h : Declaration of the CDataPoints collection class
//
// Copyright (c) 2002 Silicon Energy Corp.  All rights reserved.
// This code may be freely distributed and used in commercial or non-commercial
// applications.  However, it must be accompanied by the above copyright notice
// and this comment block.
//
// Revision History:
// WHO				WHEN				WHAT
// Bob Gunion		April 2002			Created
//

#pragma once
#include "resource.h"       // main symbols

#include "SiEDataPoints_Unmanaged.h"

#include <vector>
typedef std::vector<CAdapt<CComPtr<IDataPoint> > > DataPointVector;
typedef DataPointVector::iterator DataPointIterator;
#include <algorithm>

// CDataPoints

class ATL_NO_VTABLE CDataPoints : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CDataPoints, &CLSID_DataPoints>,
	public IDispatchImpl<IDataPoints, &__uuidof(IDataPoints), &LIBID_SiEDataPointInterfacesLib, /* wMajor = */ 1, /* wMinor = */ 0>
{
public:
	CDataPoints()
	{
	}

	DECLARE_REGISTRY_RESOURCEID(IDR_DATAPOINTS)


	BEGIN_COM_MAP(CDataPoints)
		COM_INTERFACE_ENTRY(IDispatch)
		COM_INTERFACE_ENTRY(IDataPoints)
	END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease() 
	{
		// Ensure that m_coll gets emptied
		m_coll.clear();
	}

public:
	// Override of less-than operator:  used by std::sort and std::equal_range.
	// Compares only the names of the datapoints.
	static bool operator <(const CAdapt<CComPtr<IDataPoint> > & refLHS, const CAdapt<CComPtr<IDataPoint> > & refRHS);


	// IDataPoints Methods
public:
	STDMETHOD(get_Item)(long lIndex, IDataPoint * * ppPoint);
	STDMETHOD(get_Count)(long * pVal);
	STDMETHOD(get_Capacity)(long * pVal);
	STDMETHOD(put_Capacity)(long pVal);
	STDMETHOD(Add)(IDataPoint * pPoint);
	STDMETHOD(Sort)();
	STDMETHOD(FindPoints)(BSTR bstrName, long lChannel, IDataPoints * * ppPoints);

private:
	DataPointVector				m_coll;

};

OBJECT_ENTRY_AUTO(__uuidof(DataPoints), CDataPoints)
