Attribute VB_Name = "StopWatch"
Option Explicit

' StopWatch uses the QueryPerformancexx API functions to get the CPU counts
Private Declare Function QueryPerformanceCounter Lib "kernel32" (lpPerformanceCount As Double) As Long
Private Declare Function QueryPerformanceFrequency Lib "kernel32" (lpFrequency As Double) As Long

' Since QueryPerformancexx fill a LARGE_INTEGER but VB can't pass such a thing, we need RtlMoveMemory
' to copy the returned double to our own LARGE_INTEGER.
Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (Destination As Any, Source As Any, ByVal Length As Long)

Private Type LARGE_INTEGER
    LowPart As Long
    HighPart As Long
End Type

Private m_dbStartCounter As Double
Private m_dtStart As Date

Public Sub StartTimer()
    m_dtStart = Now
    m_dbStartCounter = GetCounter
End Sub

Private Function GetCounter() As Double
    ' Get the (LARGE_INTEGER) count into a temp space
    Dim dbCounter As Double
    QueryPerformanceCounter dbCounter
    
    ' Copy the count into a usable variable
    Dim liStart As LARGE_INTEGER
    CopyMemory liStart, dbCounter, LenB(dbCounter)
    
    ' Calculate the actual count as a double
    GetCounter = (liStart.HighPart * 4294967296#) + liStart.LowPart
End Function

Public Function StopTimer() As Double
    ' Get the current counter as the stop time
    Dim dbStop As Double
    dbStop = GetCounter
    Dim dtStop As Date
    dtStop = Now
    
    ' Get the frequency of the count, in counts/second
    Dim dbFreq As Double
    Dim liFreq As LARGE_INTEGER
    QueryPerformanceFrequency dbFreq
    
    ' Copy the frequency to a usable variable
    CopyMemory liFreq, dbFreq, LenB(dbFreq)
    
    ' Copy the frequency back to dbfreq as the real number
    dbFreq = (liFreq.HighPart * 4294967296#) + liFreq.LowPart
    
    ' Use low-resolution timer if time is > 10 seconds; high-res otherwise
    If ((CDbl(dtStop) - CDbl(m_dtStart)) * 1440# * 60#) > 10# Then
        StopTimer = ((CDbl(dtStop) - CDbl(m_dtStart)) * 1440# * 60#)
    Else
        StopTimer = (dbStop - m_dbStartCounter) / dbFreq
    End If
End Function

