// StrategySpace.cpp: implementation of the CStrategySpace class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "StrategySpace.h"

#include "BitArray.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CStrategySpace::CStrategySpace(int m, int T, char stype)
{
	int i,j;

	m_iM = m;					//store parameters
	m_iT = T;
	m_iP = (int)pow(2,m_iM);	//# possible history bit strings
	
	switch (stype) {
	case 'F':					//full strategy space
		fss();			
		break;
	case 'R':					//reduced strategy space
		rss();
		break;
	default:
		throw "Invalid strategy space type";
		break;
	}

/*
	for(i=0;i<m_iQ;i++) {
		for(j=0;j<m_iP;j++)
			cout << m_SSpace[j]->GetBit(i) << " ";
		cout << endl;
	}
*/

	//create virtual points array
	m_Points = new CBitArray*[m_iQ];
	m_iPoints = new int[m_iQ];
	m_iSindex = 0;
	for(i=0;i<m_iQ;i++) {
		m_Points[i] = new CBitArray(m_iT);
		m_iPoints[i] = 0;
	}
}

CStrategySpace::~CStrategySpace()
{
	int i;

	for(i=0;i<m_iP;i++)
		delete [] m_SSpace[i];
	delete [] m_SSpace;
	
	for(i=0;i<m_iQ;i++)
		delete [] m_Points[i];
	delete [] m_Points;
	
	delete [] m_iPoints;
}


//Create full strategy space
void CStrategySpace::fss()
{
	int i,j,k,d,e;
	int Q2;

	m_iQ = (int)pow(2,m_iP);
	Q2=m_iQ/2;

	//m_SSpace[history bit][strategy number]
	m_SSpace = new CBitArray*[m_iP];
	for(i=0;i<m_iP;i++)
		m_SSpace[i] = new CBitArray(m_iQ);

	
	d = 1;
	e = 1;
	for(i=1;i<m_iP;i++) {
		for(j=d;j<Q2;j+=2*d)
			for(k=0;k<d;k++)
				m_SSpace[m_iP-e]->SetBit(j+k);
		d *= 2;
		e ++;
	}


	//and copy into bottom half
	for(i=0;i<m_iP;i++)
		for(j=0;j<Q2;j++)
			if(m_SSpace[i]->GetBit(j)==0)
				m_SSpace[i]->SetBit(j+Q2);

	return;
}

//Create reduced strategy space
void CStrategySpace::rss()
{
	int i,j,k;
	int d = 1;

	m_iQ = (int)pow(2,m_iM+1);

	//m_SSpace[history bit][strategy number]
	m_SSpace = new CBitArray*[m_iP];
	for(i=0;i<m_iP;i++)
		m_SSpace[i] = new CBitArray(m_iQ);

	m_SSpace[1]->SetBit(1);
	for(i=1;i<m_iM;i++) {
		d *= 2;
		for(j=0;j<d;j++) {
			for(k=0;k<d;k++) {
				if(m_SSpace[j]->GetBit(k)==1) {
					m_SSpace[j+d]->SetBit(k);
					m_SSpace[j+d]->SetBit(k+d);
				}
				else 
					m_SSpace[j]->SetBit(k+d);
			}
		}
	}
	d *= 2;

	//and mirror into bottom half
	for(i=0;i<d;i++)
		for(j=0;j<d;j++)
			if(m_SSpace[i]->GetBit(j)==0)
				m_SSpace[i]->SetBit(j+d);

	return;
}

//size of strategy space
const int CStrategySpace::ssize()
{	
	return m_iQ;
}

//prediction of i-th strategy
const int CStrategySpace::pred(int index, int mu)
{
	return m_SSpace[mu]->GetBit(index);
}

//strategy points of i-th strategy
const int CStrategySpace::points(int index)
{
	return m_iPoints[index];
}

//update strategy points
void CStrategySpace::points_update(int mu, int win)
{
	int i;
	int Si;
	int prediction;

	for(i=0;i<m_iQ;i++) {

		prediction = pred(i, mu);	//strategy prediction

		if(win==prediction)
			Si = 1;
		else
			Si = 0;

		m_iPoints[i] += Si - m_Points[i]->GetBit(m_iSindex);

		if(Si==1)
			m_Points[i]->SetBit(m_iSindex);
		else
			m_Points[i]->ClearBit(m_iSindex);
	}
	
	//update index - wraparound routine
	if( ++m_iSindex == m_iT)
		m_iSindex=0;

	return;

}

