#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "cgbinding.h"


#ifdef SERVER
#else
#include "Cg/cgD3D.h"
static cgDirect3D cg;
#endif


/**
   @brief Convert ascii string into VertexBindingType

   This method is the key routine that allows for the ability to run-time dynamically
   bind a Cg program to your application.  Because Cg is a general purpose programming
   language you can name your variables anything you want.  However, each one of these
   variables must be bound to your application so they can be updated int real-time.

   Therefore each variable must be mapped to a specific application operation.
   If you create new variable name that must be bound to your application you should
   add it to the VertexBindingType enumeration and provide the ASCII mapping
   for it here.  First make sure it doesn't map to an existing type, and if it does
   simply make it an alias.  Realize that every single VertexBindingType is going
   to require application code on your part to provide that service.

   @param bind_name    Ascii name of the bind variable.

   @see enum VertexBindingType

   @author John W. Ratcliff
*/
VertexBindingType CgBinding::AsciiToVertexBindingType(const char *bind_name)
{

  VertexBindingType ret = VBT_UNDEFINED;

  if ( stricmp("World",bind_name) == 0 )                               ret = VBT_WORLD;
  if ( stricmp("worldMatrix",bind_name) == 0 )                         ret = VBT_WORLD;
  if ( stricmp("object_matrix",bind_name) == 0 )                       ret = VBT_WORLD;

  if ( stricmp("WorldIT",bind_name) == 0 )                             ret = VBT_WORLDIT;
  if ( stricmp("worldInvTransMatrix",bind_name) == 0 )                 ret = VBT_WORLDIT;

  if ( stricmp("WorldView",bind_name) == 0 )                           ret = VBT_WORLDVIEW;
  if ( stricmp("ModelView",bind_name) == 0 )                           ret = VBT_WORLDVIEW;
  if ( stricmp("modelview_matrix",bind_name) == 0 )                    ret = VBT_WORLDVIEW;


  if ( stricmp("WorldViewIT",bind_name) == 0 )                         ret = VBT_WORLDVIEWIT;
  if ( stricmp("ModelViewIT",bind_name) == 0 )                         ret = VBT_WORLDVIEWIT;

  if ( stricmp("WorldViewProj",bind_name) == 0 )                       ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("ModelViewProj",bind_name) == 0 )                       ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("wvpMatrix",bind_name) == 0 )                           ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("mat",bind_name) == 0 )                                 ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("modelViewProjection",bind_name) == 0 )                 ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("obj2proj",bind_name) == 0 )                            ret = VBT_WORLDVIEWPROJ;
  if ( stricmp("objviewproj_matrix",bind_name) == 0 )                  ret = VBT_WORLDVIEWPROJ;

  if ( stricmp("projection_matrix",bind_name) == 0 )                   ret = VBT_PROJECTION_MATRIX;

  if ( stricmp("EyePosition",bind_name) == 0 )                         ret = VBT_EYEPOSITION;
  if ( stricmp("EyePos",bind_name) == 0 )                              ret = VBT_EYEPOSITION;
  if ( stricmp("eyePosition",bind_name) == 0 )                         ret = VBT_EYEPOSITION;
  if ( stricmp("eyePoint",bind_name) == 0 )                            ret = VBT_EYEPOSITION;


  if ( stricmp("ambientcolor",bind_name) == 0 )                        ret = VBT_AMBIENTCOLOR;

  if ( stricmp("LightColor",bind_name) == 0 )                          ret = VBT_LIGHTCOLOR;
  if ( stricmp("lightcolor",bind_name) == 0 )                          ret = VBT_LIGHTCOLOR;
  if ( stricmp("color",bind_name) == 0 )                               ret = VBT_LIGHTCOLOR;


  if ( stricmp("LightDir",bind_name) == 0 )                            ret = VBT_LIGHTVECTOR;
  if ( stricmp("LightVec",bind_name) == 0 )                            ret = VBT_LIGHTVECTOR;
  if ( stricmp("LightVector",bind_name) == 0 )                         ret = VBT_LIGHTVECTOR;
  if ( stricmp("lightdir",bind_name) == 0 )                            ret = VBT_LIGHTVECTOR;
  if ( stricmp("lightVector",bind_name) == 0 )                         ret = VBT_LIGHTVECTOR;


  if ( stricmp("LightPosition",bind_name) == 0 )                       ret = VBT_LIGHTPOSITION;
  if ( stricmp("LightPos",bind_name) == 0 )                            ret = VBT_LIGHTPOSITION;

  if ( stricmp("LightType",bind_name) == 0 )                           ret = VBT_LIGHTTYPE;

  if ( stricmp("SinTime",bind_name) == 0 )                             ret = VBT_TIME;
  if ( stricmp("Time",bind_name) == 0 )                                ret = VBT_TIME;

  if ( stricmp("Displace",bind_name) == 0 )                            ret = VBT_DISPLACEMENT;
  if ( stricmp("Displacement",bind_name) == 0 )                        ret = VBT_DISPLACEMENT;

  if ( stricmp("I",bind_name) == 0 )                                   ret = VBT_IN;
  if ( stricmp("IN",bind_name) == 0 )                                  ret = VBT_IN;


  if ( stricmp("Bones",bind_name) == 0 )                               ret = VBT_BONES;
  if ( stricmp("BumpScale",bind_name) == 0 )                           ret = VBT_BUMPSCALE;
  if ( stricmp("C_FOGPARAMS",bind_name) == 0 )                         ret = VBT_C_FOGPARAMS;
  if ( stricmp("C_HEIGHT_FOG_PARAMS",bind_name) == 0 )                 ret = VBT_C_HEIGHT_FOG_PARAMS;
  if ( stricmp("C_LIGHT_CONST",bind_name) == 0 )                       ret = VBT_C_LIGHT_CONST;
  if ( stricmp("C_LIGHT_POS_OSPACE",bind_name) == 0 )                  ret = VBT_C_LIGHT_POS_OSPACE;
  if ( stricmp("C_ZEROONE",bind_name) == 0 )                           ret = VBT_C_ZEROONE;
  if ( stricmp("Constants",bind_name) == 0 )                           ret = VBT_CONSTANTS;
  if ( stricmp("Consts_0512",bind_name) == 0 )                         ret = VBT_CONSTS_0512;
  if ( stricmp("Eta",bind_name) == 0 )                                 ret = VBT_ETA;
  if ( stricmp("Factors",bind_name) == 0 )                             ret = VBT_FACTORS;
  if ( stricmp("Fatness",bind_name) == 0 )                             ret = VBT_FATNESS;
  if ( stricmp("Fresnel",bind_name) == 0 )                             ret = VBT_FRESNEL;
  if ( stricmp("FresnelConstants",bind_name) == 0 )                    ret = VBT_FRESNELCONSTANTS;
  if ( stricmp("Kd",bind_name) == 0 )                                  ret = VBT_KD;
  if ( stricmp("NoiseMatrix",bind_name) == 0 )                         ret = VBT_NOISEMATRIX;
  if ( stricmp("ObjToCubeSpace",bind_name) == 0 )                      ret = VBT_OBJTOCUBESPACE;
  if ( stricmp("PIs",bind_name) == 0 )                                 ret = VBT_PIS;
  if ( stricmp("ShadowExtrudeDist",bind_name) == 0 )                   ret = VBT_SHADOWEXTRUDEDIST;
  if ( stricmp("TexTransform",bind_name) == 0 )                        ret = VBT_TEXTRANSFORM;
  if ( stricmp("TextureMat",bind_name) == 0 )                          ret = VBT_TEXTUREMATRIX;
  if ( stricmp("TextureMatrix",bind_name) == 0 )                       ret = VBT_TEXTUREMATRIX;
  if ( stricmp("VecCos",bind_name) == 0 )                              ret = VBT_VECCOS;
  if ( stricmp("VecSin",bind_name) == 0 )                              ret = VBT_VECSIN;
  if ( stricmp("WVP_0",bind_name) == 0 )                               ret = VBT_WVP_0;
  if ( stricmp("WVP_1",bind_name) == 0 )                               ret = VBT_WVP_1;
  if ( stricmp("WVP_2",bind_name) == 0 )                               ret = VBT_WVP_2;
  if ( stricmp("WVP_3",bind_name) == 0 )                               ret = VBT_WVP_3;
  if ( stricmp("Wave1",bind_name) == 0 )                               ret = VBT_WAVE1;
  if ( stricmp("Wave1Origin",bind_name) == 0 )                         ret = VBT_WAVE1ORIGIN;
  if ( stricmp("Wave2",bind_name) == 0 )                               ret = VBT_WAVE2;
  if ( stricmp("Wave2Origin",bind_name) == 0 )                         ret = VBT_WAVE2ORIGIN;
  if ( stricmp("WavesH",bind_name) == 0 )                              ret = VBT_WAVESH;
  if ( stricmp("WavesX",bind_name) == 0 )                              ret = VBT_WAVESX;
  if ( stricmp("WavesY",bind_name) == 0 )                              ret = VBT_WAVESY;
  if ( stricmp("clampZ",bind_name) == 0 )                              ret = VBT_CLAMPZ;
  if ( stricmp("freq_amp",bind_name) == 0 )                            ret = VBT_FREQ_AMP;
  if ( stricmp("furHeightScale",bind_name) == 0 )                      ret = VBT_FURHEIGHTSCALE;
  if ( stricmp("noiseMatrix",bind_name) == 0 )                         ret = VBT_NOISEMATRIX;
  if ( stricmp("pg",bind_name) == 0 )                                  ret = VBT_PG;
  if ( stricmp("scaleStart",bind_name) == 0 )                          ret = VBT_SCALESTART;
  if ( stricmp("terrain_param",bind_name) == 0 )                       ret = VBT_TERRAIN_PARAM;
  if ( stricmp("tex_offset",bind_name) == 0 )                          ret = VBT_TEX_OFFSET;
  if ( stricmp("tex_scale",bind_name) == 0 )                           ret = VBT_TEX_SCALE;
  if ( stricmp("vA",bind_name) == 0 )                                  ret = VBT_VA;
  if ( stricmp("vCos",bind_name) == 0 )                                ret = VBT_VCOS;
  if ( stricmp("vD",bind_name) == 0 )                                  ret = VBT_VD;
  if ( stricmp("vSin",bind_name) == 0 )                                ret = VBT_VSIN;

  return ret;

}

/**
   @brief Convert ascii string into PixelBindingType

   This method is the key routine that allows for the ability to run-time dynamically
   bind a Cg program to your application.  Because Cg is a general purpose programming
   language you can name your variables anything you want.  However, each one of these
   variables must be bound to your application so they can be updated int real-time.

   Therefore each variable must be mapped to a specific application operation.
   If you create new variable name that must be bound to your application you should
   add it to the VertexBindingType enumeration and provide the ASCII mapping
   for it here.  First make sure it doesn't map to an existing type, and if it does
   simply make it an alias.  Realize that every single PixelBindingType is going
   to require application code on your part to provide that service.

   The only reason that PixelBindings and VertexBindings are seperated out is because
   they are such distinctly different operations I felt it would create less confusion
   than if they were all mixed up into a single enumeration.

   @param bind_name    Ascii name of the bind variable.

   @see enum PixelBindingType

   @author John W. Ratcliff
*/
PixelBindingType CgBinding::AsciiToPixelBindingType(const char *bind_name)
{

  PixelBindingType ret = PBT_UNDEFINED;


  if ( stricmp(bind_name,"tex0") == 0 )        ret = PBT_TEX0;
  if ( stricmp(bind_name,"tex1") == 0 )        ret = PBT_TEX1;
  if ( stricmp(bind_name,"tex2") == 0 )        ret = PBT_TEX2;
  if ( stricmp(bind_name,"tex3") == 0 )        ret = PBT_TEX3;
  if ( stricmp(bind_name,"tex4") == 0 )        ret = PBT_TEX4;
  if ( stricmp(bind_name,"tex5") == 0 )        ret = PBT_TEX5;
  if ( stricmp(bind_name,"tex6") == 0 )        ret = PBT_TEX6;
  if ( stricmp(bind_name,"tex7") == 0 )        ret = PBT_TEX7;
  if ( stricmp(bind_name,"height") ==0)        ret = PBT_HEIGHT;
  if ( stricmp(bind_name,"base_color0") == 0 ) ret = PBT_BASE_COLOR0;
  if ( stricmp(bind_name,"base_color1") == 0 ) ret = PBT_BASE_COLOR1;
  if ( stricmp("I",bind_name) == 0 )           ret = PBT_IN;
  if ( stricmp("IN",bind_name) == 0 )          ret = PBT_IN;


  return ret;

}


/**
   @brief Constructor for CgBinding interface.

   Simply initializes member variables to their default state.

   @author John W. Ratcliff
*/
CgBinding::CgBinding(void)
{
  mGuid = 0;
  mContextContainer = 0;
  strcpy(mLastError,"NO ERROR: CgBind Initialized");
  mLastListing = 0;
}

/**
   @brief Destructor for the CgBinding class

   Free's up all previously allocated shader programs.

   @author John W. Ratcliff
*/
CgBinding::~CgBinding(void)
{
  if ( 1 )
  {
    CgVertexShaderMap::iterator i;
    for (i=mVertexShaders.begin(); i!=mVertexShaders.end(); ++i)
    {
      CgVertexShader *cgs = (*i).second;
      delete cgs;
    }
  }
  if ( 1 )
  {
    CgPixelShaderMap::iterator i;
    for (i=mPixelShaders.begin(); i!=mPixelShaders.end(); ++i)
    {
      CgPixelShader *cgs = (*i).second;
      delete cgs;
    }
  }
}


/**
   @brief Create a vertex shader program by compiling a Cg source file.

   This method creates a vertex shader on the current device by loading
   the Cg source code specified.  The Cg source file will be compiled
   and bound to the vertex format specified by the application.  If
   there are any errors it will return a null pointer and you can
   find out why it failed by calling the method GetLastError.

   You can get the Cg compiler listing output by calling
   GetLastListing


   @param program        The ASCII name of the Cg program to compile.
   @param type           Specifies what vertex format we want to bind the vertex shader to.
   @param device_handle  The Direct3d device handle.

   @see enum VertexBindingFormat

   @author John W. Ratcliff
*/
CgVertexShader * CgBinding::CreateVertexShader(const char *program,VertexBindingFormat type,unsigned int device_handle)
{
#ifdef SERVER
  return 0;
#else
  CgVertexShader *shader = 0;

  if ( !mContextContainer ) // first time in
  {
    cg.AddFilePath(".");
    cg.AddFilePath("..");
    LPDIRECT3DDEVICE8 device = (LPDIRECT3DDEVICE8) device_handle; // Cast to a direct3d device handle
    cgContextContainer * cgc = cg.CreateContextContainer( device );
    mContextContainer = (void *) cgc;
  }

  if ( mContextContainer )
  {
    cgContextContainer * cgc = (cgContextContainer *) mContextContainer;
    cgVertexAttribute *cgva = (cgVertexAttribute *) GetVertexAttributes(type);
    if ( cgva )
    {

      cgProgramContainer *pVertexProgramContainer = cgc->LoadCGProgramFromFile(program, program, cgDX8VertexProfile, cgva );

      if ( pVertexProgramContainer )
      {
        unsigned int handle = (unsigned int) pVertexProgramContainer;


        cgBindIter *bind_point = 0;

        //cgProgramIter * program_iter = pContextContainer->ProgramByName(program);

        cgProgramIter * program_iter = cgc->GetNextProgram(0);

        assert( program_iter );

        if ( program_iter )
        {

          mGuid++;
          shader = new CgVertexShader(mGuid,type,handle);

          bool ok = true; // by default, sweet success

          do
          {

            bind_point = cgGetNextBind(program_iter,bind_point);


            if ( bind_point )
            {

              const char *key = cgGetBindParamName(bind_point);

              assert( key );

              int len = strlen(key);

              char *scratch = new char[len+1];

              strcpy(scratch,key);

              char *prefix  = scratch;
              char *postfix = scratch;

              while ( *postfix )
              {
                if ( *postfix == '.' )
                {
                  *postfix = 0;
                  postfix++;
                  break;
                }
                postfix++;
              }

              if ( strlen(postfix) )
              {

                if ( stricmp(prefix,"main") == 0 )
                {
                  VertexBindingType type = AsciiToVertexBindingType(postfix);

                  if ( type == VBT_UNDEFINED )
                  {
                    assert(0);
                    sprintf(mLastError,"Encountered unknown bind name (%s).  Can't run this shader till you make a valid bind format type for it.\n",postfix);
                    ok = false;
                    bind_point = 0; // force it to exit the iteration
                  }
                  else
                  {
                    ok = ValidateVertexBindingType(type); // can the application handle this?
                    if ( ok )
                    {

                      cgBindIter *bind_key = pVertexProgramContainer->GetParameterBindByName(postfix);

                      // *NOTE* the bind_key is *NOT* the same pointer as the bind_iterator we are iterating through.
                      //        I don't know why necessarily that is, but it is that way!
                      assert( bind_key );

                      if ( bind_key )
                      {
                        unsigned int bind_handle = (unsigned int) bind_key; // convert to a generic handle.
                        shader->AddBinding(type,bind_handle);
                      }
                      else
                      {
                        sprintf(mLastError,"Bizarre, while iterating though (%s) we were unable to lookup a valid bind_iterator to key (%s)\n",key,postfix);
                        ok = false;
                        bind_point = 0;
                      }

                    }
                    else
                    {
                      sprintf(mLastError,"Warning: Application is unable to handle binding type (%s).  If you want to run this shader you will need to provide an API hool for this bind name.\n",postfix);
                      ok = true; // we have to treat this only as a warning, however the shader may not function correctly.
                    }
                  }

                }
              }
              delete scratch;
            }
          } while ( bind_point );

          if ( ok )
          {
            mVertexShaders[mGuid] = shader; // new shader created!
          }
          else
          {
            delete shader;
            shader = 0;
          }

        }
        else
        {
          sprintf(mLastError,"Bizarre error, was unable to get a program iterator to a valid loaded program (%s)\n",program);
        }
      }
      else
      {
        sprintf(mLastError,"Failed to load .Cg program file %s.  Examine listing file for more information.\n",program);
      }

      mLastListing = cgc->GetLastListing();
    }
    else
    {
      sprintf(mLastError,"Failed to resolve VertexFormat passed to CgBinding::CreateVertexShader. Type: %d FILE: %s LINE: %d\n",type,__FILE__,__LINE__);
    }
  }
  else
  {
    sprintf(mLastError,"Unable to create context container.");
  }


  return shader;
#endif
}

/**
   @brief Release a previously loaded VertexShader program.

   @param shader         Pointer to the previously successfully loaded vertex shader program.

   @author John W. Ratcliff
*/
bool  CgBinding::ReleaseVertexShader(CgVertexShader *shader)
{
  bool ret = false;

  int guid = shader->GetGuid();
  CgVertexShaderMap::iterator found;
  found = mVertexShaders.find(guid);
  if ( found == mVertexShaders.end() )
  {
    sprintf(mLastError,"Tried to release a VertexShader that has an invalid Guid.  Bad pointer??\n");
    assert(0);
  }
  else
  {
    delete shader;
    mVertexShaders.erase(found);
    ret = true;
  }
  return ret;
}

/**
   @brief Converts the enumerated vertex type into a valid vertex attributes pointer.

   Again, this is just a convenience method so that all of our supported vertex formats
   can reside in one centralized piece of code.  This routine accepts the enumerated
   vertex format type and converts it into a proper vertex definition pointer to be
   passed down into the Cg program loader routine.

   @param type           Specifies what vertex format we want to bind the vertex shader to.

   @see enum VertexBindingFormat

   @author John W. Ratcliff
*/
void * CgBinding::GetVertexAttributes(VertexBindingFormat type)
{
  void * ret = 0;

#ifdef SERVER
#else

  switch ( type )
  {
    case VBF_FORMAT0:
      if ( 1 )
      {
        static cgVertexAttribute vertex_attributes[] =
        {
            // number of floats for each element
            {2, "temp", 0},  // dummy
            {2, "position", 0},
            {2, "texcoord0", 0},
            {2, "texcoord1", 0},
            {0, 0, 0}
        };

        ret = vertex_attributes;
      }
      break;
    case VBF_FORMAT1:
      if ( 1 )
      {
        static cgVertexAttribute vertex_attributes[] =
        {
            // number of floats for each element
            {3, "Position", 0},
            {3, "Normal"   , 0},
            {2, "TexCoord0", 0},
            {0, 0, 0}
        };

        ret = vertex_attributes;
      }
      break;
  }
#endif
  return ret;
}

/**
   @brief Make this vertex shader program active on the hardware.

   @author John W. Ratcliff
*/
bool CgVertexShader::Activate(void)
{
  bool ok = true;
#ifdef SERVER
#else
  cgProgramContainer *program = (cgProgramContainer *) mHandle;
  program->SetShaderActive(); // make it active
#endif
  return ok;
}

/**
   @brief Make this pixel shader program active on the hardware.

   @author John W. Ratcliff
*/
bool CgPixelShader::Activate(void)
{
  bool ok = true;
#ifdef SERVER
#else
  cgProgramContainer *program = (cgProgramContainer *) mHandle;
  if ( program )
  {
    program->SetShaderActive(); // make it active
  }
#endif
  return ok;
}

/**
   @brief Applies all of the dynamic bindings between the application and the vertex shader program.

   This method iterates through all of the run-time bindings and performs a callback for each to
   the application to get the current data.

   @param bindings       A pointer to the base class that contains the virtual methods we need to retrieve data from the application.
   @param context1       An optional pointer provided by the application to help resolve any contextual requirements.
   @param context2       A second optional pointer provided by the application to resolve contextual requirements.

   @see class CgVertexBind

   @author John W. Ratcliff
*/
bool CgVertexShader::ActivateBindings(CgBinding *bindings,void *context1,void *context2)
{
  bool ok = true;

  CgVertexBindVector::iterator i;
  for (i=mBindings.begin(); i!=mBindings.end(); ++i)
  {
    (*i).Activate(bindings,mHandle,context1,context2);
  }

  return ok;
}

/**
   @brief Applies all of the dynamic bindings between the application and the pixel shader program

   This method iterates through all of the run-time bindings and performs a callback for each to
   the application to get the current data values.

   @param bindings       A pointer to the base class that contains the virtual methods we need to retrieve data from the application.
   @param context1       An optional pointer provided by the application to help resolve any contextual requirements.
   @param context2       A second optional pointer provided by the application to resolve contextual requirements.

   @see class CgPixelBind

   @author John W. Ratcliff
*/
bool CgPixelShader::ActivateBindings(CgBinding *bindings,void *context1,void *context2)
{
  bool ok = true;

  CgPixelBindVector::iterator i;
  for (i=mBindings.begin(); i!=mBindings.end(); ++i)
  {
    (*i).Activate(bindings,mHandle,context1,context2);
  }

  return ok;
}

/**
   @brief Passes the correct value down to the vertex shader program for a single dynamic bind point.

   The application provides the current valid data item for this binding type.  If the application returns
   a null pointer then the value is not updated.

   @param bindings       A pointer to the base class that contains the virtual methods we need to retrieve data from the application.
   @param program_handle A handle to the CgShader program we are binding to.
   @param context1       An optional pointer provided by the application to help resolve any contextual requirements.
   @param context2       A second optional pointer provided by the application to resolve contextual requirements.

   @see enum CgBindingType

   @author John W. Ratcliff
*/
bool CgVertexBind::Activate(CgBinding *bindings,unsigned int program_handle,void *context1,void *context2)
{
  bool ok = false;
#ifdef SERVER
#else
  void *data = bindings->GetVertexBindingType(mBind,context1,context2);

  if ( data )
  {
    cgBindIter *bind_point = (cgBindIter *) mHandle;
    cgProgramContainer *program = (cgProgramContainer *) program_handle;
    program->SetShaderConstant( bind_point, data );
    ok = true;
  }
#endif
  return ok;
}



/**
   @brief Passes the correct value down to the pixel shader program for a single dynamic bind point.

   The application provides the current valid data item for this binding type.  If the application returns
   a null pointer then the value is not updated unless it is an enumeated texture type.  The binding types
   PBT_TEX0 through PBT_TEX7 are reserved for updating texture arguments.

   @param bindings       A pointer to the base class that contains the virtual methods we need to retrieve data from the application.
   @param program_handle A handle to the CgShader program we are binding to.
   @param context1       An optional pointer provided by the application to help resolve any contextual requirements.
   @param context2       A second optional pointer provided by the application to resolve contextual requirements.

   @see enum CgBindingType

   @author John W. Ratcliff
*/
bool CgPixelBind::Activate(CgBinding *bindings,unsigned int program_handle,void *context1,void *context2)
{
  bool ok = false;
#ifdef SERVER
#else
  void *data = bindings->GetPixelBindingType(mBind,context1,context2);

  cgBindIter *bind_point = (cgBindIter *) mHandle;
  cgProgramContainer *program = (cgProgramContainer *) program_handle;

  switch ( mBind )
  {
    case PBT_TEX0:
    case PBT_TEX1:
    case PBT_TEX2:
    case PBT_TEX4:
    case PBT_TEX5:
    case PBT_TEX6:
    case PBT_TEX7:
      program->SetTexture( bind_point, (LPDIRECT3DTEXTURE8)data );
      break;
    default:
      if ( data ) program->SetShaderConstant( bind_point, data );
      break;
  }

  ok = true;
#endif

  return ok;
}


/**
   @brief Create a pixel shader program by compiling a Cg source file.

   This method creates a pixel shader on the current device by loading
   the Cg source code specified. If there are any errors it will return a
   null pointer and you can find out why it failed by calling the method
   GetLastError.

   You can get the Cg compiler listing output by calling
   GetLastListing


   @param program        The ASCII name of the Cg program to compile.
   @param device_handle  The Direct3d device handle.

   @author John W. Ratcliff
*/
CgPixelShader * CgBinding::CreatePixelShader(const char *program,unsigned int device_handle)
{
#ifdef SERVER
  return 0;
#else
  CgPixelShader *shader = 0;

  if ( !mContextContainer ) // first time in
  {
    cg.AddFilePath(".");
    cg.AddFilePath("..");
    LPDIRECT3DDEVICE8 device = (LPDIRECT3DDEVICE8) device_handle; // Cast to a direct3d device handle
    cgContextContainer * cgc = cg.CreateContextContainer( device );
    mContextContainer = (void *) cgc;
  }

  if ( mContextContainer )
  {
    cgContextContainer * cgc = (cgContextContainer *) mContextContainer;
    if ( 1 )
    {

      cgProgramContainer *pPixelProgramContainer = cgc->LoadCGProgramFromFile(program, program, cgDX8PixelProfile, 0 );

      if ( pPixelProgramContainer )
      {
        unsigned int handle = (unsigned int) pPixelProgramContainer;


        cgBindIter *bind_point = 0;

        //cgProgramIter * program_iter = pContextContainer->ProgramByName(program);

        cgProgramIter * program_iter = cgc->GetNextProgram(0);

        assert( program_iter );

        if ( program_iter )
        {

          mGuid++;
          shader = new CgPixelShader(mGuid,handle);

          bool ok = true; // by default, sweet success

          do
          {

            bind_point = cgGetNextBind(program_iter,bind_point);


            if ( bind_point )
            {

              const char *key = cgGetBindParamName(bind_point);

              assert( key );

              int len = strlen(key);

              char *scratch = new char[len+1];

              strcpy(scratch,key);

              char *prefix  = scratch;
              char *postfix = scratch;

              while ( *postfix )
              {
                if ( *postfix == '.' )
                {
                  *postfix = 0;
                  postfix++;
                  break;
                }
                postfix++;
              }

              if ( strlen(postfix) )
              {

                if ( stricmp(prefix,"main") == 0 )
                {
                  PixelBindingType type = AsciiToPixelBindingType(postfix);

                  if ( type == VBT_UNDEFINED )
                  {
                    assert(0);
                    sprintf(mLastError,"Encountered unknown bind name (%s).  Can't run this shader till you make a valid bind format type for it.\n",postfix);
                    ok = false;
                    bind_point = 0; // force it to exit the iteration
                  }
                  else
                  {
                    if ( 1 )
                    {

                      cgBindIter *bind_key = pPixelProgramContainer->GetParameterBindByName(postfix);

                      // *NOTE* the bind_key is *NOT* the same pointer as the bind_iterator we are iterating through.
                      //        I don't know why necessarily that is, but it is that way!
                      assert( bind_key );

                      if ( bind_key )
                      {
                        unsigned int bind_handle = (unsigned int) bind_key; // convert to a generic handle.
                        shader->AddBinding(type,bind_handle);
                      }
                      else
                      {
                        sprintf(mLastError,"Bizarre, while iterating though (%s) we were unable to lookup a valid bind_iterator to key (%s)\n",key,postfix);
                        ok = false;
                        bind_point = 0;
                      }

                    }
                    else
                    {
                      sprintf(mLastError,"Warning: Application is unable to handle binding type (%s).  If you want to run this shader you will need to provide an API hool for this bind name.\n",postfix);
                      ok = true; // we have to treat this only as a warning, however the shader may not function correctly.
                    }
                  }

                }
              }
              delete scratch;
            }
          } while ( bind_point );

          if ( ok )
          {
            mPixelShaders[mGuid] = shader; // new shader created!
          }
          else
          {
            delete shader;
            shader = 0;
          }

        }
        else
        {
          sprintf(mLastError,"Bizarre error, was unable to get a program iterator to a valid loaded program (%s)\n",program);
        }
      }
      else
      {
        sprintf(mLastError,"Failed to load .Cg program file %s.  Examine listing file for more information.\n",program);
      }

      mLastListing = cgc->GetLastListing();
    }
  }
  else
  {
    sprintf(mLastError,"Unable to create context container.");
  }


  return shader;
#endif
}

/**
   @brief Release a previously loaded VertexShader program.

   @param shader         Pointer to the previously successfully loaded vertex shader program.

   @author John W. Ratcliff
*/
bool  CgBinding::ReleasePixelShader(CgPixelShader *shader)
{
  bool ret = false;

  int guid = shader->GetGuid();
  CgPixelShaderMap::iterator found;
  found = mPixelShaders.find(guid);
  if ( found == mPixelShaders.end() )
  {
    sprintf(mLastError,"Tried to release a PixelShader that has an invalid Guid.  Bad pointer??\n");
    assert(0);
  }
  else
  {
    delete shader;
    mPixelShaders.erase(found);
    ret = true;
  }
  return ret;
}

