#ifndef CGBINDINGTYPE_H

#define CGBINDINGTYPE_H

/**
   @enum VertexBindingFormat

   @brief A single enumeration to specify all vertex formats your application creates shaders for.

   This enumeration exists simply so we can create a single logical binding point for the huge
   permutation of vertex format specifcations possible.  Currently under DirectX 8.1 you must have
   a different vertex shader for every vertex format.  Fortuantely, this limitiation is going to be fixed under
   DirectX 9.0.

   @author John W. Ratcliff

   @date  July 18, 2002

*/
enum VertexBindingFormat
{
  VBF_FORMAT0,            /* Used by ripple shader. */
  VBF_FORMAT1,            /* Most standard vertex format float3, float3, float2, position, normal, texcoord0 */
  VBF_LAST
};



/**
   @enum VertexBindingType

   @brief A single enumeration for translating all ascii keywords in a vertexshader into logical operations

   This Enumeration lists all of the logical key bindings in a large collection of .Cg scripts.  If you need a new
   key binding for your Cg program you would add it here.  If your script aliases one of these existing keyword bindings
   then you would simply add it to the 'AsciiToVertexBindingType' method.

   @author John W. Ratcliff

   @date  July 18, 2002

*/
enum VertexBindingType
{
  VBT_UNDEFINED,                 /* unresolved type */

  VBT_WORLD,                     /* object to world matrix */
  VBT_WORLDIT,                   /* inverse transform of object to world */
  VBT_WORLDVIEW,                 /* world to view matrix */
  VBT_WORLDVIEWIT,               /* inverse transform of world to view */
  VBT_WORLDVIEWPROJ,             /* world view projection matrix */
  VBT_PROJECTION_MATRIX,         /* just the projection matrix */

  VBT_EYEPOSITION,               /* position of the eye in world space */

  VBT_AMBIENTCOLOR,              /* ambient color */
  VBT_LIGHTCOLOR,                /* color of the light */
  VBT_LIGHTPOSITION,             /* light position */
  VBT_LIGHTVECTOR,               /* world space unit vector direction of light */
  VBT_LIGHTTYPE,                 /* light type */

  VBT_WAVESH,                    /* Wave height */
  VBT_WAVESX,                    /* number of waves in the x direction */
  VBT_WAVESY,                    /* number of waves in the y direction */

  VBT_TIME,                      /* current time value */

  VBT_IN,
  VBT_KD,


  VBT_BONES,
  VBT_BUMPSCALE,
  VBT_C_FOGPARAMS,
  VBT_C_HEIGHT_FOG_PARAMS,
  VBT_C_LIGHT_CONST,
  VBT_C_LIGHT_POS_OSPACE,
  VBT_C_ZEROONE,
  VBT_CLAMPZ,
  VBT_CONSTANTS,
  VBT_CONSTS_0512,
  VBT_DISPLACEMENT,
  VBT_ETA,
  VBT_FACTORS,
  VBT_FATNESS,
  VBT_FREQ_AMP,
  VBT_FRESNEL,
  VBT_FRESNELCONSTANTS,
  VBT_FURHEIGHTSCALE,


  VBT_NOISEMATRIX,

  VBT_OBJTOCUBESPACE,
  VBT_PG,
  VBT_PIS,
  VBT_SCALESTART,
  VBT_SHADOWEXTRUDEDIST,
  VBT_TERRAIN_PARAM,

  VBT_TEX_OFFSET,
  VBT_TEX_SCALE,
  VBT_TEXTRANSFORM,
  VBT_TEXTUREMATRIX,


  VBT_VA,
  VBT_VCOS,
  VBT_VD,
  VBT_VECCOS,
  VBT_VECSIN,
  VBT_VSIN,

  VBT_WAVE1,
  VBT_WAVE1ORIGIN,

  VBT_WAVE2,
  VBT_WAVE2ORIGIN,


  VBT_WVP_0,
  VBT_WVP_1,
  VBT_WVP_2,
  VBT_WVP_3,

  VBT_LAST
};

/**
   @enum PixelBindingType

   @brief A single enumeration for translating all ascii keywords in a PixelShader into logical operations

   This Enumeration lists all of the logical key bindings in a large collection of .Cg scripts.  If you need a new
   key binding for your Cg program you would add it here.  If your script aliases one of these existing keyword bindings
   then you would simply add it to the 'AsciiToVertexBindingType' method.

   @note  PBT_TEX0 through PBT_TEX7 are reserved bindings to indicate texture binds, not data fielsd.

   @author John W. Ratcliff

   @date  July 18, 2002

*/
enum PixelBindingType
{
  PBT_UNDEFINED,                 // unresolved type
  PBT_IN,
  PBT_TEX0,
  PBT_TEX1,
  PBT_TEX2,
  PBT_TEX3,
  PBT_TEX4,
  PBT_TEX5,
  PBT_TEX6,
  PBT_TEX7,
  PBT_HEIGHT,
  PBT_BASE_COLOR0,
  PBT_BASE_COLOR1,
  PBT_LAST
};

#endif
