

#include "stdafx.h"

// Definitions for ran1
#define IA 16807
#define IM 2147483647
#define AM1 (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV1 (1+(IM-1)/NTAB)
//#define EPS 1.2e-7
#define RNMX (1.0-EPS)
double ran1(long *idum)
{
	int j;
	long k;
	static long iy=0;
	static long iv[NTAB];
	float temp;
	if (*idum <= 0 || !iy) 
	{ 
		if (-(*idum) < 1) 
			*idum=1;		//Be sure to prevent idum = 0.
		else 
			*idum = -(*idum);
		for (j=NTAB+7;j>=0;j--) 
		{ 
			k=(*idum)/IQ;
			*idum=IA*(*idum-k*IQ)-IR*k;
			if (*idum < 0) 
				*idum += IM;
			if (j < NTAB) 
				iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ;	//Start here when not initializing.
	*idum=IA*(*idum-k*IQ)-IR*k; 
	if (*idum < 0)
		*idum += IM;
	j=iy/NDIV1; 
	iy=iv[j]; 
	iv[j] = *idum;
	if ((temp=AM1*iy) > RNMX) 
		return RNMX;			//Because users don't expect endpoint values.
	else 
		return temp;
}


// Definitions for ran2
#define IM1 2147483563
#define IM2 2147483399
#define AM (1.0/IM1)
#define IMM1 (IM1-1)
#define IA1 40014
#define IA2 40692
#define IQ1 53668
#define IQ2 52774
#define IR1 12211
#define IR2 3791
//#define NTAB 32
#define NDIV (1+IMM1/NTAB)
//#define EPS 1.2e-7
//#define RNMX (1.0-EPS)


/* 
// L'Ecuyer's uniform random number generator with Bays-Durham shuffle.
// Reference : Press, W. H., et al.,
//	       Numerical Recipes in C.
*/
double ran2(long *idum)
{
	
	int j;
	long k;
	static long idum2=123456789;
	static long iy=0;
	static long iv[NTAB];
	double temp;

	if(*idum<=0) 
	{
		if (-(*idum)<1) *idum=1;
		else *idum = -(*idum);
		idum2=(*idum);
		for (j=NTAB+7;j>=0;j--)
		{
			k=(*idum)/IQ1;
			*idum=IA1*(*idum-k*IQ1)-k*IR1;
			if (*idum<0) *idum+=IM1;
			if (j<NTAB) iv[j] = *idum;
		}
		iy=iv[0];
	}
	k=(*idum)/IQ1;
	*idum=IA1*(*idum-k*IQ1)-k*IR1;
	if (*idum<0) *idum += IM1;
	k=idum2/IQ2;
	idum2=IA2*(idum2-k*IQ2)-k*IR2;
	if (idum2<0) idum2 += IM2;
	j=iy/NDIV;
	iy=iv[j]-idum2;
	iv[j]=*idum;
	if (iy<1) iy+=IMM1;
	if ((temp=AM*iy) > RNMX) return RNMX;
	else return temp;
}

/*
// Standard normal random number generator using
// rejection method with Box-Muller transformation. 
// Calls L'Ecuyer's uniform random number generator.
// Reference : Press, W. H., et al.,
//             Numerical Recipes in C.
*/



//Returns a normally distributed deviate with zero mean and unit variance
// using ran2(idum) as the source of uniform deviates

double gasdev(long *idum)
{
	//float ran2(long *idum);
	static int iset=0;
	static float gset;
	double fac,rsq,v1,v2;

	if(iset==0)
	{
		do
		{
			v1=2.0*ran2(idum)-1.0;
			v2=2.0*ran2(idum)-1.0;
			rsq=v1*v1+v2*v2;
		}
		while (rsq>=1.0 || rsq == 0.0);
		fac=sqrt(-2.0*log(rsq)/rsq);
		gset=v1*fac;
		iset=1;
		return v2*fac;
	}
	else
	{
		iset=0;
		return gset;
	}
}


// Returns a deviate distributed as a gamma distribution of integer order ia, i.e., a waiting time
// to the iath event in a Poisson process of unit mean, using ran1(idum) as the source of
// uniform deviates.
double gamdev(int ia, long *idum)
{
	double ran1(long *idum);
	void nrerror(char error_text[]);
	int j;
	float am,e,s,v1,v2,x,y;
	if (ia < 1) 
		//nrerror("Error in routine gamdev");
		throw 10;
	if(ia < 6) 
	{					//Use direct method, adding waiting times. 
		x=1.0;
		for (j=1;j<=ia;j++) 
			x *= ran1(idum);
		x = -log(x);
	}
	else 
	{					//Use rejection method.
		do {
			do {
				do {	//These four lines generate the tangent of a random angle.
					v1=ran1(idum);
					v2=2.0*ran1(idum)-1.0;
				}
				while (v1*v1+v2*v2 > 1.0); 
				y=v2/v1;
				am=ia-1;
				s=sqrt(2.0*am+1.0);
				x=s*y+am; 
			}
			while (x <= 0.0);		//Reject in region of zero probability.
			e=(1.0+y*y)*exp(am*log(x/am)-s*y); //Ratio of prob. fn. to comparison fn.
		}
		while (ran1(idum) > e); //Reject on basis of a second uniform deviate. 
	}
	return x;

}

//return a uniformly distributed random number between l and u
int unidev(int l, int u, long *idum)
{
	return (int)(l + (u-l+1)*ran2(idum));
}

double max(double a, double b)
{
	if (a>b) return(a);
	else return(b);
}

