/*
 *   Cort Dougan <cort@fsmlabs.com>
 *
 * __RTLINUX_COPYRIGHT__
 */

#include <rtl.h>
#include <pthread.h>
#include <rtl_sched.h>
#include <rtl_time.h>
#include <rtl_fifo.h>
#include <rtl_profile.h>
#include <rtl_conf.h>
#include <rtl_core.h>
#include <rtl_sync.h>
#include <rtl_posixio.h>
#include <unistd.h>

MODULE_AUTHOR("Cort Dougan <cort@fsmlabs.com>");
MODULE_DESCRIPTION("Parallel Port Driver");

#define PORT 0x378

char out_byte;

static ssize_t rtl_par_read(struct rtl_file *filp, char *buf, size_t count,
			    off_t* ppos)
{
	if ( count < sizeof(char) )
		return -1;

	buf[0] = inb( PORT );
	
	return 0;
}

static ssize_t rtl_par_write(struct rtl_file *filp, const char *buf,
			     size_t count, off_t* ppos)
{
	int i;
	
	for ( i = 0; i < count; i++ )
	{
		out_byte = buf[i];
		outb( out_byte, PORT );
	}
	
	return 0;
}

static int rtl_par_ioctl(struct rtl_file *filp,
			 unsigned int request, unsigned long l)
{
	switch ( request )
	{
	case RTL_PAR_SETBIT:
		out_byte |= 1<<l;
		break;
	case RTL_PAR_CLEARBIT:
		out_byte &= ~(1<<l);
		break;
	default:
		return -EINVAL;
	}

	outb( out_byte, PORT );

	return 0;
}

static int rtl_par_open(struct rtl_file *filp)
{
	return 0;
}

static int rtl_par_release(struct rtl_file *filp)
{
	return 0;
}

static struct rtl_file_operations rtl_par_fops =
{
       	NULL,
	rtl_par_read,
	rtl_par_write,
	rtl_par_ioctl,
	NULL,
	rtl_par_open,
	rtl_par_release
};

int init_module(void)
{
	if ( rtl_register_dev( "lpt", &rtl_par_fops ) )
	{
		printk("Unable to install Parallel Port Driver\n");
		return -EIO;
	}
	
	return 0;
}

void cleanup_module(void)
{
	rtl_unregister_dev( "lpt" );
}
